/*
 * Copyright (c) 1999 Thomas Nystrm and Stacken Computer Club
 * (Royal Institute of Technology, Stockholm, Sweden).
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Kungliga Tekniska
 *      Hgskolan and its contributors.
 * 
 * 4. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "common.h"
#pragma hdrstop

#include <ktftp.h>
#include "sfilter.h"
#include "ftphnd.h"
#include "ftpsim.h"

FTPSim::FTPSim(TWindow *Parent, SetupParam *sp, DebugParam *dp,
	TWindow *_LogWindow)
	: FTPHandler(Parent, sp, dp, _LogWindow)
{
}

FTPSim::~FTPSim()
{
}

bool
FTPSim::Abort(bool CheckTerminate)
{
	return FTPHandler::Abort(CheckTerminate);
}

void
FTPSim::Close()
{
}

bool
FTPSim::Login()
{
	LogTest("Login called\n");
	return true;
}

bool
FTPSim::Logout()
{
	LogTest("Logout called\n");
	return true;
}

ListFilter *
FTPSim::GetFilter()
{
	return new UnixFilter();
}

const char *
FTPSim::GetCwd()
{
	return "/afs/stacken.kth.se/home/thn";
}

bool
FTPSim::MkDir(const char *Dst)
{
	LogTest("MkDir called <%s>\n", Dst);
	return false;
}

bool
FTPSim::RmDir(const char *Dst)
{
	LogTest("RmDir called <%s>\n", Dst);
	return false;
}

bool
FTPSim::Rm(const char *Dst)
{
	LogTest("Rm called <%s>\n", Dst);
	return false;
}

bool
FTPSim::GetFile(const char *RemFile, const char *LclFile)
{
	LogTest("GetFile called <%s> <%s>\n", RemFile, LclFile);
	return false;
}

bool
FTPSim::PutFile(const char *RemFile, const char *LclFile)
{
	LogTest("PutFile called <%s> <%s>\n", RemFile, LclFile);
	return false;
}

bool
FTPSim::RemoteKauth(const char *principal, const char *password)
{
	LogTest("RemoteKauth called <%s> <%s>\n", principal, password);
	return false;
}

bool
FTPSim::RemoteKlist()
{
	LogTest("Klist called\n");
	return false;
}

bool
FTPSim::RemoteKdestroy()
{
	LogTest("Kdestroy called\n");
	return false;
}

BufferedStream *
FTPSim::GetList(const char *path)
{
	char FileName[MAX_PATH];
	char buf[512];
	HANDLE f;
	DWORD bufsz;
	bool EndOfFile = false;
	BufferedStream *stream;
	char *cp;

	if (strncmp(path, "/afs/stacken.kth.se/home/thn", 28) != 0) {
		LogTest("Not a valid path\n");
		return NULL;
		}

	strcpy_truncate(FileName, "c:\\winp\\krb\\sim\\root",
		sizeof(FileName));
	strcat_truncate(FileName, &path[28], sizeof(FileName));
	for (cp = FileName; *cp; cp++) {
		if (*cp == '/')
			*cp = '.';
		}

	LogTest("Reading from file: <%s>\n", FileName);
	
	if ((f = CreateFile(FileName, GENERIC_READ, 0, NULL, OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL, 0)) == INVALID_HANDLE_VALUE) {
		LogText("Can't open file: %s\n", Wstrerror(::GetLastError()));
		return NULL;
		}

	
	stream = new BufferedStream();
	
	while (ReadFile(f, buf, sizeof(buf), &bufsz, 0)) {
		if (bufsz == 0) {
			EndOfFile = true;
			break;
			}
		stream->PutChars((uchar *)buf, bufsz);
		}
	
	if (!EndOfFile) {
		LogText("Read error from file: %s\n", 
			Wstrerror(::GetLastError()));
		CloseHandle(f);
		delete stream;
		return NULL;
		}
	
	LogTest("File read\n");
	CloseHandle(f);
	return stream;
}

bool
FTPSim::TransferEnd()
{
	return false;
}

bool
FTPSim::Rename(const char *CurrentPath, const char *OldName,
	const char *NewName)
{
	LogTest("Rename <%s> <%s> <%s>\n",
		CurrentPath, OldName, NewName);
	
	return true;
}

bool
FTPSim::Delete(const char *CurrentPath, const char *File)
{
	LogTest("Delete <%s> <%s>\n", CurrentPath, File);
	
	return true;
}

bool
FTPSim::IsLocal()
{
	return true;
}

bool
FTPSim::IsOpen()
{
	return true;
}

#pragma argsused
bool
FTPSim::Open(const char *hostname, const char *username,
	const char *PortStr)
{
	return true;
}

void
FTPSim::SendAbort()
{
}

